@@ -1,9 +1,9 @@
 /*
- * "$Id: cgi.h 7721 2008-07-11 22:48:49Z mike $"
+ * "$Id: cgi.h 8243 2009-01-14 00:44:55Z mike $"
  *
  *   CGI support library definitions.
  *
- *   Copyright 2007 by Apple Inc.
+ *   Copyright 2007-2009 by Apple Inc.
  *   Copyright 1997-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -59,6 +59,7 @@ extern void		cgiCopyTemplateFile(FILE *out, const char *tmpl);
 extern void		cgiCopyTemplateLang(const char *tmpl);
 extern int		cgiDoSearch(void *search, const char *text);
 extern void		cgiEndHTML(void);
+extern void		cgiEndMultipart(void);
 extern char		*cgiFormEncode(char *dst, const char *src, size_t dstsize);
 extern void		cgiFreeSearch(void *search);
 extern const char	*cgiGetArray(const char *name, int element);
@@ -72,6 +73,8 @@ extern const char	*cgiGetVariable(const char *name);
 extern int		cgiInitialize(void);
 extern int		cgiIsPOST(void);
 extern void		cgiMoveJobs(http_t *http, const char *dest, int job_id);
+extern void		cgiPrintCommand(http_t *http, const char *dest,
+			                const char *command, const char *title);
 extern void		cgiPrintTestPage(http_t *http, const char *dest);
 extern char		*cgiRewriteURL(const char *uri, char *url, int urlsize,
 			               const char *newresource);
@@ -91,10 +94,12 @@ extern void		cgiSetVariable(const char *name, const char *value);
 extern void		cgiShowIPPError(const char *message);
 extern void		cgiShowJobs(http_t *http, const char *dest);
 extern void		cgiStartHTML(const char *title);
+extern void		cgiStartMultipart(void);
+extern int		cgiSupportsMultipart(void);
 extern const char	*cgiText(const char *message);
 
 #endif /* !_CUPS_CGI_H_ */
 
 /*
- * End of "$Id: cgi.h 7721 2008-07-11 22:48:49Z mike $".
+ * End of "$Id: cgi.h 8243 2009-01-14 00:44:55Z mike $".
  */
