@@ -1,9 +1,9 @@
 /*
- * "$Id: pstops.c 7721 2008-07-11 22:48:49Z mike $"
+ * "$Id: pstops.c 8515 2009-04-17 03:18:14Z mike $"
  *
  *   PostScript filter for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 2007-2008 by Apple Inc.
+ *   Copyright 2007-2009 by Apple Inc.
  *   Copyright 1993-2007 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -16,32 +16,36 @@
  *
  * Contents:
  *
- *   main()               - Main entry...
- *   add_page()           - Add a page to the pages array...
+ *   main()               - Main entry.
+ *   add_page()           - Add a page to the pages array.
  *   cancel_job()         - Flag the job as canceled.
  *   check_range()        - Check to see if the current page is selected for
- *   copy_bytes()         - Copy bytes from the input file to stdout...
- *   copy_comments()      - Copy all of the comments section...
- *   copy_dsc()           - Copy a DSC-conforming document...
- *   copy_non_dsc()       - Copy a document that does not conform to the DSC...
- *   copy_page()          - Copy a page description...
- *   copy_prolog()        - Copy the document prolog section...
- *   copy_setup()         - Copy the document setup section...
- *   copy_trailer()       - Copy the document trailer...
- *   do_prolog()          - Send the necessary document prolog commands...
- *   do_setup()           - Send the necessary document setup commands...
+ *                          printing.
+ *   copy_bytes()         - Copy bytes from the input file to stdout.
+ *   copy_comments()      - Copy all of the comments section.
+ *   copy_dsc()           - Copy a DSC-conforming document.
+ *   copy_non_dsc()       - Copy a document that does not conform to the DSC.
+ *   copy_page()          - Copy a page description.
+ *   copy_prolog()        - Copy the document prolog section.
+ *   copy_setup()         - Copy the document setup section.
+ *   copy_trailer()       - Copy the document trailer.
+ *   do_prolog()          - Send the necessary document prolog commands.
+ *   do_setup()           - Send the necessary document setup commands.
  *   doc_printf()         - Send a formatted string to stdout and/or the temp
  *                          file.
  *   doc_puts()           - Send a nul-terminated string to stdout and/or the
  *                          temp file.
  *   doc_write()          - Send data to stdout and/or the temp file.
- *   end_nup()            - End processing for N-up printing...
+ *   end_nup()            - End processing for N-up printing.
  *   include_feature()    - Include a printer option/feature command.
- *   parse_text()         - Parse a text value in a comment...
- *   set_pstops_options() - Set pstops options...
- *   skip_page()          - Skip past a page that won't be printed...
- *   start_nup()          - Start processing for N-up printing...
+ *   parse_text()         - Parse a text value in a comment.
+ *   set_pstops_options() - Set pstops options.
+ *   skip_page()          - Skip past a page that won't be printed.
+ *   start_nup()          - Start processing for N-up printing.
+ *   write_label_prolog() - Write the prolog with the classification and page
+ *                          label.
  *   write_labels()       - Write the actual page labels.
+ *   write_options()      - Write options provided via %%IncludeFeature.
  */
 
 /*
@@ -211,10 +215,12 @@ static void		write_label_prolog(pstops_doc_t *doc, const char *label,
 			                   float bottom, float top,
 					   float width);
 static void		write_labels(pstops_doc_t *doc, int orient);
+static void		write_options(pstops_doc_t  *doc, ppd_file_t *ppd,
+			              int num_options, cups_option_t *options);
 
 
 /*
- * 'main()' - Main entry...
+ * 'main()' - Main entry.
  */
 
 int					/* O - Exit status */
@@ -245,8 +251,9 @@ main(int  argc,				/* I - Number of command-line args */
 
   if (argc < 6 || argc > 7)
   {
-    fprintf(stderr, _("Usage: %s job-id user title copies options [file]\n"),
-            argv[0]);
+    _cupsLangPrintf(stderr,
+                    _("Usage: %s job-id user title copies options [file]\n"),
+                    argv[0]);
     return (1);
   }
 
@@ -281,8 +288,8 @@ main(int  argc,				/* I - Number of command-line args */
 
     if ((fp = cupsFileOpen(argv[6], "r")) == NULL)
     {
-      fprintf(stderr, _("ERROR: Unable to open file \"%s\" - %s\n"),
-              argv[6], strerror(errno));
+      _cupsLangPrintf(stderr, _("ERROR: Unable to open file \"%s\" - %s\n"),
+                      argv[6], strerror(errno));
       return (1);
     }
   }
@@ -293,7 +300,7 @@ main(int  argc,				/* I - Number of command-line args */
 
   if ((len = cupsFileGetLine(fp, line, sizeof(line))) == 0)
   {
-    fputs(_("ERROR: Empty print file!\n"), stderr);
+    _cupsLangPuts(stderr, _("ERROR: Empty print file!\n"));
     return (1);
   }
 
@@ -412,7 +419,7 @@ main(int  argc,				/* I - Number of command-line args */
 
 
 /*
- * 'add_page()' - Add a page to the pages array...
+ * 'add_page()' - Add a page to the pages array.
  */
 
 static pstops_page_t *			/* O - New page info object */
@@ -427,15 +434,17 @@ add_page(pstops_doc_t *doc,		/* I - Document information */
 
   if (!doc->pages)
   {
-    fprintf(stderr, _("EMERG: Unable to allocate memory for pages array: %s\n"),
-            strerror(errno));
+    _cupsLangPrintf(stderr,
+                    _("EMERG: Unable to allocate memory for pages array: %s\n"),
+                    strerror(errno));
     exit(1);
   }
 
   if ((pageinfo = calloc(1, sizeof(pstops_page_t))) == NULL)
   {
-    fprintf(stderr, _("EMERG: Unable to allocate memory for page info: %s\n"),
-            strerror(errno));
+    _cupsLangPrintf(stderr,
+                    _("EMERG: Unable to allocate memory for page info: %s\n"),
+                    strerror(errno));
     exit(1);
   }
 
@@ -530,7 +539,7 @@ check_range(pstops_doc_t *doc,		/* I - Document information */
 
 
 /*
- * 'copy_bytes()' - Copy bytes from the input file to stdout...
+ * 'copy_bytes()' - Copy bytes from the input file to stdout.
  */
 
 static void
@@ -547,13 +556,13 @@ copy_bytes(cups_file_t *fp,		/* I - File to read from */
 
   if (cupsFileSeek(fp, offset) < 0)
   {
-    fprintf(stderr,
+    _cupsLangPrintf(stderr,
 #ifdef HAVE_LONG_LONG
-            _("ERROR: Unable to seek to offset %lld in file - %s\n"),
+		    _("ERROR: Unable to seek to offset %lld in file - %s\n"),
 #else
-            _("ERROR: Unable to seek to offset %ld in file - %s\n"),
+		    _("ERROR: Unable to seek to offset %ld in file - %s\n"),
 #endif /* HAVE_LONG_LONG */
-            CUPS_LLCAST offset, strerror(errno));
+		    CUPS_LLCAST offset, strerror(errno));
     return;
   }
 
@@ -575,7 +584,7 @@ copy_bytes(cups_file_t *fp,		/* I - File to read from */
 
 
 /*
- * 'copy_comments()' - Copy all of the comments section...
+ * 'copy_comments()' - Copy all of the comments section.
  *
  * This function expects "line" to be filled with a comment line.
  * On return, "line" will contain the next line in the file, if any.
@@ -636,7 +645,7 @@ copy_comments(cups_file_t  *fp,		/* I - File to read from */
 
 
       if (saw_pages)
-        fputs(_("ERROR: Duplicate %%Pages: comment seen!\n"), stderr);
+        _cupsLangPuts(stderr, _("ERROR: Duplicate %%Pages: comment seen!\n"));
 
       saw_pages = 1;
 
@@ -683,7 +692,8 @@ copy_comments(cups_file_t  *fp,		/* I - File to read from */
     else if (!strncmp(line, "%%BoundingBox:", 14))
     {
       if (saw_bounding_box)
-        fputs(_("ERROR: Duplicate %%BoundingBox: comment seen!\n"), stderr);
+        _cupsLangPuts(stderr,
+	              _("ERROR: Duplicate %%BoundingBox: comment seen!\n"));
       else if (strstr(line + 14, "(atend)"))
       {
        /*
@@ -694,7 +704,7 @@ copy_comments(cups_file_t  *fp,		/* I - File to read from */
 	              doc->bounding_box + 1, doc->bounding_box + 2,
 		      doc->bounding_box + 3) != 4)
       {
-	fputs(_("ERROR: Bad %%BoundingBox: comment seen!\n"), stderr);
+	_cupsLangPuts(stderr, _("ERROR: Bad %%BoundingBox: comment seen!\n"));
 
 	doc->bounding_box[0] = (int)PageLeft;
 	doc->bounding_box[1] = (int)PageBottom;
@@ -746,10 +756,10 @@ copy_comments(cups_file_t  *fp,		/* I - File to read from */
   }
 
   if (!saw_bounding_box)
-    fputs(_("ERROR: No %%BoundingBox: comment in header!\n"), stderr);
+    _cupsLangPuts(stderr, _("ERROR: No %%BoundingBox: comment in header!\n"));
 
   if (!saw_pages)
-    fputs(_("ERROR: No %%Pages: comment in header!\n"), stderr);
+    _cupsLangPuts(stderr, _("ERROR: No %%Pages: comment in header!\n"));
 
   if (!saw_for)
     WriteTextComment("For", doc->user);
@@ -799,7 +809,7 @@ copy_comments(cups_file_t  *fp,		/* I - File to read from */
 
 
 /*
- * 'copy_dsc()' - Copy a DSC-conforming document...
+ * 'copy_dsc()' - Copy a DSC-conforming document.
  *
  * This function expects "line" to be filled with the %!PS-Adobe comment line.
  */
@@ -1059,7 +1069,7 @@ copy_dsc(cups_file_t  *fp,		/* I - File to read from */
 
 
 /*
- * 'copy_non_dsc()' - Copy a document that does not conform to the DSC...
+ * 'copy_non_dsc()' - Copy a document that does not conform to the DSC.
  *
  * This function expects "line" to be filled with the %! comment line.
  */
@@ -1082,8 +1092,10 @@ copy_non_dsc(cups_file_t  *fp,		/* I - File to read from */
   * that may not print correctly...
   */
 
-  fputs(_("WARNING: This document does not conform to the Adobe Document "
-          "Structuring Conventions and may not print correctly!\n"), stderr);
+  _cupsLangPuts(stderr,
+                _("WARNING: This document does not conform to the Adobe "
+		  "Document Structuring Conventions and may not print "
+		  "correctly!\n"));
 
  /*
   * Then write a standard DSC comment section...
@@ -1239,7 +1251,7 @@ copy_non_dsc(cups_file_t  *fp,		/* I - File to read from */
 
 
 /*
- * 'copy_page()' - Copy a page description...
+ * 'copy_page()' - Copy a page description.
  *
  * This function expects "line" to be filled with a %%Page comment line.
  * On return, "line" will contain the next line in the file, if any.
@@ -1271,13 +1283,13 @@ copy_page(cups_file_t  *fp,		/* I - File to read from */
 
   if (!parse_text(line + 7, &ptr, label, sizeof(label)))
   {
-    fputs(_("ERROR: Bad %%Page: comment in file!\n"), stderr);
+    _cupsLangPuts(stderr, _("ERROR: Bad %%Page: comment in file!\n"));
     label[0] = '\0';
     number   = doc->page;
   }
   else if (strtol(ptr, &ptr, 10) == LONG_MAX || !isspace(*ptr & 255))
   {
-    fputs(_("ERROR: Bad %%Page: comment in file!\n"), stderr);
+    _cupsLangPuts(stderr, _("ERROR: Bad %%Page: comment in file!\n"));
     number = doc->page;
   }
 
@@ -1347,7 +1359,8 @@ copy_page(cups_file_t  *fp,		/* I - File to read from */
                  bounding_box + 1, bounding_box + 2,
 		 bounding_box + 3) != 4)
       {
-        fputs(_("ERROR: Bad %%PageBoundingBox: comment in file!\n"), stderr);
+        _cupsLangPuts(stderr,
+	              _("ERROR: Bad %%PageBoundingBox: comment in file!\n"));
         memcpy(bounding_box, doc->bounding_box,
 	       sizeof(bounding_box));
       }
@@ -1565,53 +1578,7 @@ copy_page(cups_file_t  *fp,		/* I - File to read from */
 
 
     if (pageinfo->num_options > 0)
-    {
-      int		i;		/* Looping var */
-      ppd_option_t	*option;	/* PPD option */
-      int		min_order;	/* Minimum OrderDependency value */
-      char		*doc_setup,	/* DocumentSetup commands to send */
-			*any_setup;	/* AnySetup commands to send */
-
-
-     /*
-      * Yes, figure out the minimum OrderDependency value...
-      */
-
-      if ((option = ppdFindOption(ppd, "PageRegion")) != NULL)
-	min_order = option->order;
-      else
-	min_order = 999.0f;
-
-      for (i = 0; i < pageinfo->num_options; i ++)
-	if ((option = ppdFindOption(ppd, pageinfo->options[i].name)) != NULL &&
-            option->order < min_order)
-          min_order = option->order;
-
-     /*
-      * Mark and extract them...
-      */
-
-      cupsMarkOptions(ppd, pageinfo->num_options, pageinfo->options);
-
-      doc_setup = ppdEmitString(ppd, PPD_ORDER_DOCUMENT, min_order);
-      any_setup = ppdEmitString(ppd, PPD_ORDER_ANY, min_order);
-
-     /*
-      * Then send them out...
-      */
-
-      if (doc_setup)
-      {
-	doc_puts(doc, doc_setup);
-	free(doc_setup);
-      }
-
-      if (any_setup)
-      {
-	doc_puts(doc, any_setup);
-	free(any_setup);
-      }
-    }
+      write_options(doc, ppd, pageinfo->num_options, pageinfo->options);
 
    /*
     * Output commands for the current page...
@@ -1757,7 +1724,7 @@ copy_page(cups_file_t  *fp,		/* I - File to read from */
 
 
 /*
- * 'copy_prolog()' - Copy the document prolog section...
+ * 'copy_prolog()' - Copy the document prolog section.
  *
  * This function expects "line" to be filled with a %%BeginProlog comment line.
  * On return, "line" will contain the next line in the file, if any.
@@ -1801,7 +1768,7 @@ copy_prolog(cups_file_t  *fp,		/* I - File to read from */
     if (!strncmp(line, "%%EndProlog", 11))
       linelen = cupsFileGetLine(fp, line, linesize);
     else
-      fputs(_("ERROR: Missing %%EndProlog!\n"), stderr);
+      _cupsLangPuts(stderr, _("ERROR: Missing %%EndProlog!\n"));
   }
 
   doc_puts(doc, "%%EndProlog\n");
@@ -1811,7 +1778,7 @@ copy_prolog(cups_file_t  *fp,		/* I - File to read from */
 
 
 /*
- * 'copy_setup()' - Copy the document setup section...
+ * 'copy_setup()' - Copy the document setup section.
  *
  * This function expects "line" to be filled with a %%BeginSetup comment line.
  * On return, "line" will contain the next line in the file, if any.
@@ -1825,6 +1792,10 @@ copy_setup(cups_file_t  *fp,		/* I - File to read from */
 	   ssize_t      linelen,	/* I - Length of initial line */
 	   size_t       linesize)	/* I - Size of line buffer */
 {
+  int		num_options;		/* Number of options */
+  cups_option_t	*options;		/* Options */
+
+
   while (strncmp(line, "%%BeginSetup", 12))
   {
     if (!strncmp(line, "%%Page:", 7))
@@ -1838,6 +1809,11 @@ copy_setup(cups_file_t  *fp,		/* I - File to read from */
 
   doc_puts(doc, "%%BeginSetup\n");
   
+  do_setup(doc, ppd);
+
+  num_options = 0;
+  options     = NULL;
+
   if (!strncmp(line, "%%BeginSetup", 12))
   {
     while (strncmp(line, "%%EndSetup", 10))
@@ -1851,8 +1827,7 @@ copy_setup(cups_file_t  *fp,		/* I - File to read from */
 	*/
 
         if (doc->number_up == 1 && !doc->fitplot)
-	  doc->num_options = include_feature(ppd, line, doc->num_options,
-                                             &(doc->options));
+	  num_options = include_feature(ppd, line, num_options, &options);
       }
       else if (strncmp(line, "%%BeginSetup", 12))
         doc_write(doc, line, linelen);
@@ -1864,10 +1839,14 @@ copy_setup(cups_file_t  *fp,		/* I - File to read from */
     if (!strncmp(line, "%%EndSetup", 10))
       linelen = cupsFileGetLine(fp, line, linesize);
     else
-      fputs(_("ERROR: Missing %%EndSetup!\n"), stderr);
+      _cupsLangPuts(stderr, _("ERROR: Missing %%EndSetup!\n"));
   }
 
-  do_setup(doc, ppd);
+  if (num_options > 0)
+  {
+    write_options(doc, ppd, num_options, options);
+    cupsFreeOptions(num_options, options);
+  }
 
   doc_puts(doc, "%%EndSetup\n");
 
@@ -1876,7 +1855,7 @@ copy_setup(cups_file_t  *fp,		/* I - File to read from */
 
 
 /*
- * 'copy_trailer()' - Copy the document trailer...
+ * 'copy_trailer()' - Copy the document trailer.
  *
  * This function expects "line" to be filled with a %%Trailer comment line.
  * On return, "line" will contain the next line in the file, if any.
@@ -1925,7 +1904,7 @@ copy_trailer(cups_file_t  *fp,		/* I - File to read from */
 
 
 /*
- * 'do_prolog()' - Send the necessary document prolog commands...
+ * 'do_prolog()' - Send the necessary document prolog commands.
  */
 
 static void
@@ -1964,7 +1943,7 @@ do_prolog(pstops_doc_t *doc,		/* I - Document information */
 
 
 /*
- * 'do_setup()' - Send the necessary document setup commands...
+ * 'do_setup()' - Send the necessary document setup commands.
  */
 
 static void
@@ -1983,7 +1962,7 @@ do_setup(pstops_doc_t *doc,		/* I - Document information */
   doc_puts(doc, "userdict dup(\\004)cvn{}put (\\004\\004)cvn{}put\n");
 
  /*
-  * Mark any options from %%IncludeFeature: comments...
+  * Mark job options...
   */
 
   cupsMarkOptions(ppd, doc->num_options, doc->options);
@@ -2024,7 +2003,10 @@ do_setup(pstops_doc_t *doc,		/* I - Document information */
   */
 
   if (doc->number_up > 1)
+  {
+    doc_puts(doc, "userdict/CUPSsetpagedevice/setpagedevice load put\n");
     doc_puts(doc, "userdict/setpagedevice{pop}bind put\n");
+  }
 
  /*
   * Changes to the transfer function must be made AFTER any
@@ -2104,9 +2086,9 @@ doc_printf(pstops_doc_t *doc,		/* I - Document information */
 
   if (bytes > sizeof(buffer))
   {
-    fprintf(stderr,
-            _("ERROR: doc_printf overflow (%d bytes) detected, aborting!\n"),
-            (int)bytes);
+    _cupsLangPrintf(stderr,
+		    _("ERROR: doc_printf overflow (%d bytes) detected, "
+		      "aborting!\n"), (int)bytes);
     exit(1);
   }
 
@@ -2147,7 +2129,7 @@ doc_write(pstops_doc_t *doc,		/* I - Document information */
 
 
 /*
- * 'end_nup()' - End processing for N-up printing...
+ * 'end_nup()' - End processing for N-up printing.
  */
 
 static void
@@ -2235,7 +2217,7 @@ include_feature(
 
   if (sscanf(line + 17, "%254s%254s", name, value) != 2)
   {
-    fputs(_("ERROR: Bad %%IncludeFeature: comment!\n"), stderr);
+    _cupsLangPuts(stderr, _("ERROR: Bad %%IncludeFeature: comment!\n"));
     return (num_options);
   }
 
@@ -2245,22 +2227,23 @@ include_feature(
 
   if ((option = ppdFindOption(ppd, name + 1)) == NULL)
   {
-    fprintf(stderr, _("WARNING: Unknown option \"%s\"!\n"), name + 1);
+    _cupsLangPrintf(stderr, _("WARNING: Unknown option \"%s\"!\n"), name + 1);
     return (num_options);
   }
 
   if (option->section == PPD_ORDER_EXIT ||
       option->section == PPD_ORDER_JCL)
   {
-    fprintf(stderr, _("WARNING: Option \"%s\" cannot be included via "
-                      "IncludeFeature!\n"), name + 1);
+    _cupsLangPrintf(stderr, _("WARNING: Option \"%s\" cannot be included via "
+                              "IncludeFeature!\n"), name + 1);
     return (num_options);
   }
 
   if (!ppdFindChoice(option, value))
   {
-    fprintf(stderr, _("WARNING: Unknown choice \"%s\" for option \"%s\"!\n"),
-            value, name + 1);
+    _cupsLangPrintf(stderr,
+                    _("WARNING: Unknown choice \"%s\" for option \"%s\"!\n"),
+                    value, name + 1);
     return (num_options);
   }
 
@@ -2273,7 +2256,7 @@ include_feature(
 
 
 /*
- * 'parse_text()' - Parse a text value in a comment...
+ * 'parse_text()' - Parse a text value in a comment.
  *
  * This function parses a DSC text value as defined on page 36 of the
  * DSC specification.  Text values are either surrounded by parenthesis
@@ -2362,7 +2345,7 @@ parse_text(const char *start,		/* I - Start of text value */
 
 
 /*
- * 'set_pstops_options()' - Set pstops options...
+ * 'set_pstops_options()' - Set pstops options.
  */
 
 static void
@@ -2432,8 +2415,8 @@ set_pstops_options(
 
     if (intval < 10 || intval > 1000)
     {
-      fprintf(stderr, _("ERROR: Unsupported brightness value %s, using "
-                        "brightness=100!\n"), val);
+      _cupsLangPrintf(stderr, _("ERROR: Unsupported brightness value %s, using "
+                                "brightness=100!\n"), val);
       doc->brightness = 1.0f;
     }
     else
@@ -2477,12 +2460,14 @@ set_pstops_options(
     doc->emit_jcl = 1;
 
  /*
-  * fitplot
+  * fitplot/fit-to-page
   */
 
   if ((val = cupsGetOption("fitplot", num_options, options)) != NULL &&
-      (!strcasecmp(val, "true") || !strcasecmp(val, "on") ||
-       !strcasecmp(val, "yes")))
+      !strcasecmp(val, "true"))
+    doc->fitplot = 1;
+  else if ((val = cupsGetOption("fit-to-page", num_options, options)) != NULL &&
+           !strcasecmp(val, "true"))
     doc->fitplot = 1;
 
  /*
@@ -2499,8 +2484,8 @@ set_pstops_options(
 
     if (intval < 1 || intval > 10000)
     {
-      fprintf(stderr, _("ERROR: Unsupported gamma value %s, using "
-                        "gamma=1000!\n"), val);
+      _cupsLangPrintf(stderr, _("ERROR: Unsupported gamma value %s, using "
+                                "gamma=1000!\n"), val);
       doc->gamma = 1.0f;
     }
     else
@@ -2552,9 +2537,9 @@ set_pstops_options(
           doc->number_up = intval;
 	  break;
       default :
-          fprintf(stderr,
-	          _("ERROR: Unsupported number-up value %d, using "
-		    "number-up=1!\n"), intval);
+          _cupsLangPrintf(stderr,
+			  _("ERROR: Unsupported number-up value %d, using "
+			    "number-up=1!\n"), intval);
           doc->number_up = 1;
 	  break;
     }
@@ -2586,8 +2571,8 @@ set_pstops_options(
       doc->number_up_layout = PSTOPS_LAYOUT_BTRL;
     else
     {
-      fprintf(stderr, _("ERROR: Unsupported number-up-layout value %s, using "
-                        "number-up-layout=lrtb!\n"), val);
+      _cupsLangPrintf(stderr, _("ERROR: Unsupported number-up-layout value %s, "
+                                "using number-up-layout=lrtb!\n"), val);
       doc->number_up_layout = PSTOPS_LAYOUT_LRTB;
     }
   }
@@ -2636,8 +2621,8 @@ set_pstops_options(
       doc->page_border = PSTOPS_BORDERDOUBLE2;
     else
     {
-      fprintf(stderr, _("ERROR: Unsupported page-border value %s, using "
-                        "page-border=none!\n"), val);
+      _cupsLangPrintf(stderr, _("ERROR: Unsupported page-border value %s, "
+                                "using page-border=none!\n"), val);
       doc->page_border = PSTOPS_BORDERNONE;
     }
   }
@@ -2731,8 +2716,7 @@ set_pstops_options(
     if ((doc->temp = cupsTempFile2(doc->tempfile,
                                    sizeof(doc->tempfile))) == NULL)
     {
-      fprintf(stderr, _("ERROR: Unable to create temporary file: %s\n"),
-              strerror(errno));
+      _cupsLangPrintError(_("ERROR: Unable to create temporary file"));
       exit(1);
     }
   }
@@ -2757,7 +2741,7 @@ set_pstops_options(
 
 
 /*
- * 'skip_page()' - Skip past a page that won't be printed...
+ * 'skip_page()' - Skip past a page that won't be printed.
  */
 
 static ssize_t				/* O - Length of next line */
@@ -2819,7 +2803,7 @@ skip_page(cups_file_t *fp,		/* I - File to read from */
 
 
 /*
- * 'start_nup()' - Start processing for N-up printing...
+ * 'start_nup()' - Start processing for N-up printing.
  */
 
 static void
@@ -2844,9 +2828,6 @@ start_nup(pstops_doc_t *doc,		/* I - Document information */
   if (doc->number_up > 1)
     doc_puts(doc, "userdict/ESPsave save put\n");
 
-  if (doc->mirror)
-    doc_printf(doc, "%.1f 0.0 translate -1 1 scale\n", PageWidth);
-
   pos   = (number - 1) % doc->number_up;
   pagew = PageRight - PageLeft;
   pagel = PageTop - PageBottom;
@@ -2895,6 +2876,9 @@ start_nup(pstops_doc_t *doc,		/* I - Document information */
   else if (doc->number_up > 1 || doc->fitplot)
     doc_printf(doc, "%.1f %.1f translate\n", PageLeft, PageBottom);
 
+  if (doc->mirror)
+    doc_printf(doc, "%.1f 0.0 translate -1 1 scale\n", PageWidth);
+
   switch (doc->number_up)
   {
     default :
@@ -3411,5 +3395,82 @@ write_labels(pstops_doc_t *doc,		/* I - Document information */
 
 
 /*
- * End of "$Id: pstops.c 7721 2008-07-11 22:48:49Z mike $".
+ * 'write_options()' - Write options provided via %%IncludeFeature.
+ */
+
+static void
+write_options(
+    pstops_doc_t  *doc,		/* I - Document */
+    ppd_file_t    *ppd,		/* I - PPD file */
+    int           num_options,	/* I - Number of options */
+    cups_option_t *options)	/* I - Options */
+{
+  int		i;		/* Looping var */
+  ppd_option_t	*option;	/* PPD option */
+  int		min_order;	/* Minimum OrderDependency value */
+  char		*doc_setup,	/* DocumentSetup commands to send */
+		*any_setup;	/* AnySetup commands to send */
+
+
+ /*
+  * Figure out the minimum OrderDependency value...
+  */
+
+  if ((option = ppdFindOption(ppd, "PageRegion")) != NULL)
+    min_order = option->order;
+  else
+    min_order = 999.0f;
+
+  for (i = 0; i < num_options; i ++)
+    if ((option = ppdFindOption(ppd, options[i].name)) != NULL &&
+	option->order < min_order)
+      min_order = option->order;
+
+ /*
+  * Mark and extract them...
+  */
+
+  cupsMarkOptions(ppd, num_options, options);
+
+  doc_setup = ppdEmitString(ppd, PPD_ORDER_DOCUMENT, min_order);
+  any_setup = ppdEmitString(ppd, PPD_ORDER_ANY, min_order);
+
+ /*
+  * Then send them out...
+  */
+
+  if (doc->number_up > 1)
+  {
+   /*
+    * Temporarily restore setpagedevice so we can set the options...
+    */
+
+    doc_puts(doc, "userdict/setpagedevice/CUPSsetpagedevice load put\n");
+  }
+
+  if (doc_setup)
+  {
+    doc_puts(doc, doc_setup);
+    free(doc_setup);
+  }
+
+  if (any_setup)
+  {
+    doc_puts(doc, any_setup);
+    free(any_setup);
+  }
+
+  if (doc->number_up > 1)
+  {
+   /*
+    * Disable setpagedevice again...
+    */
+
+    doc_puts(doc, "userdict/setpagedevice{pop}bind put\n");
+  }
+}
+
+
+/*
+ * End of "$Id: pstops.c 8515 2009-04-17 03:18:14Z mike $".
  */
